<?php /*

**************************************************************************

Plugin Name:  WPhone
Plugin URI:   http://wphoneplugin.org/
Version:      1.0.1
Description:  A lightweight admin interface for iPhone and other mobile devices.
Author:       <a href="http://tekartist.org/">Stephane Daury</a>, <a href="http://literalbarrage.org/blog/">Doug Stewart</a>, and <a href="http://www.viper007bond.com/">Viper007Bond</a>

**************************************************************************/

class WPhone {
	var $version;
	var $folder;
	var $whitelist = array();
	var $browsers = array();
	var $ismobile = FALSE;
	var $iscompat = FALSE;
	var $context;
	var $current_basename;
	var $site_url;
	var $interface_url;
	var $interface_path;
	var $admin_url;
	var $compatregex;
	var $forcetakeover = FALSE;
	var $obstartstatus = FALSE;

	// Plugin initialization stuff
	function WPhone() {
		$this->version = '1.0.1';

		if ( !defined('PLUGINDIR') ) {
			add_action( 'admin_notices', array(&$this, 'WordPressTooOld') );
			return;
		}

		$this->folder = PLUGINDIR . '/wphone';

		// Make sure people install this plugin correctly by doing a couple simple checks
		if ( 'wphone' !== basename( dirname( __FILE__ ) ) || !file_exists( ABSPATH . $this->folder . '/includes/header.php' ) ) {
			add_action( 'admin_notices', array(&$this, 'IncorrectlyInstalled') );
			return;
		}

		// Load up the localization file if we're using WordPress in a different language
		// Just drop it in this plugin's "localization" folder and name it "wphone-[value in wp-config].mo"
		load_plugin_textdomain( 'wphone', $this->folder . '/localization' );

		define( 'WPHONE_USEUI_COOKIE', 'wordpress_wphone_useui_' . COOKIEHASH );
		define( 'WPHONE_CHKBOX_COOKIE', 'wordpress_wphone_chkbox_' . COOKIEHASH );

		# Register our hooks

		add_action( 'plugins_loaded', array(&$this, 'SetFilterableVariables'), 17 );
		add_action( 'plugins_loaded', array(&$this, 'LookForMobileBrowser'), 57 );

		// Login form stuff
		add_action( 'login_form', array(&$this, 'LoginFormCheckbox'), 7 );
		add_action( 'wp_login', array(&$this, 'LoginFormPOST') );
		add_action( 'wp_logout', array(&$this, 'ClearCookie') );

		# The rest are admin only, so don't bog down the main site
		if ( !is_admin() ) return;

		add_filter( 'wp_redirect', array(&$this, 'MaybeChangeRedirect'), 1 );

		add_action( 'init', array(&$this, 'MaybeStartOutputBuffering'), 1 );
		add_action( 'admin_notices', array(&$this, 'MaybeDisplayBufferError') );
		add_action( 'admin_footer', array(&$this, 'MaybeOverride'), 77 );

		// Welcome message
		add_action( 'activate_wphone/wphone.php', array(&$this, 'PluginActivated') );
		add_action( 'admin_notices', array(&$this, 'WelcomeMessage'), 7 );
		add_action( 'plugins_loaded', array(&$this, 'QuickSwitchInterface'), 17 );
	}


	// Displays an error saying this version of WordPress is unsupported
	function WordPressTooOld() { ?>

	<div id="wphone-error" class="error"><p><?php printf( __('<strong>WordPress Version Too Old:</strong> WPhone is only compatible with WordPress 2.1+ and is designed for use with the latest version of WordPress. Please <a href="%s">upgrade to the latest version</a>.', 'wphone'), 'http://wordpress.org/download/' ); ?></p></div>
<?php
	}


	// Displays an error saying this plugin is incorrectly installed
	function IncorrectlyInstalled() { ?>

	<div id="wphone-error" class="error"><p><?php printf( __("<strong>WPhone Incorrectly Installed:</strong> WPhone must be installed to <code>%s</code> and it's directory structure intact. You will not be able to use the plugin until this is fixed.", 'wphone'), '/' . PLUGINDIR . '/wphone/' ); ?></p></div>
<?php
	}


	// When this plugin is activated, redirect to a special URL
	function PluginActivated() {
		wp_redirect( 'plugins.php?activate=true&wphone=activated' );
		exit();
	}


	// Display an error if this plugin is incorrectly installed
	function WelcomeMessage() {
		if ( 'activated' != $_GET['wphone'] ) return;
?>

	<div id="wphone-welcome" class="updated fade"><p><?php printf( __('Thanks for installing <strong>WPhone</strong>! We hope your enjoy it. You can start using the admin interface by checking the new checkbox on the login form when you login on your phone.', 'wphone'), './?wphone=on' ); ?></p></div>
<?php
		//<a href="%s">right now</a> or later 
	}


	// Quick (dis|en)ables the interface on the fly by modifying the use-mobile-admin cookie and then redirects to the dashboard
	function QuickSwitchInterface() {
		if ( 'on' != $_GET['wphone'] && 'off' != $_GET['wphone'] ) return;

		if ( 'on' == $_GET['wphone'] ) $expire = time() + 31536000; // on
		else                           $expire = time() - 31536000; // off

		setcookie( WPHONE_USEUI_COOKIE, 1, $expire, COOKIEPATH, COOKIE_DOMAIN );

		wp_redirect( './' );
		exit();
	}


	// Set some variables that can be modified by other plugins
	function SetFilterableVariables() {
		$this->current_basename = apply_filters( 'wphone_currentbasename', basename( $_SERVER['PHP_SELF'] ) );

		$this->compatregex      = apply_filters( 'wphone_compatregex', '/webkit/i' ); // Used to find JS compatible mobile browsers (i.e. the iPhone)

		$this->site_url         = get_bloginfo( 'wpurl' );
		$this->interface_url    = apply_filters( 'wphone_interfaceurl', $this->site_url . '/' . $this->folder . '/includes' );
		$this->admin_url        = apply_filters( 'wphone_adminurl', $this->site_url . '/wp-admin' );


		# Most of these browser names are thanks to Alex King via his GPL'ed "WordPress Mobile Edition" plugin

		// Set and sort some browsers that are never mobile browsers
		$this->whitelist = apply_filters( 'wphone_whitelistbrowsers', array(
			'Stand Alone/QNws',
		) );
		natcasesort( $this->whitelist );

		// Set and sort some browsers that we should consider mobile browsers
		$this->browsers = apply_filters( 'wphone_mobilebrowsers', array(
			'2.0 MMP',
			'240x320',
			'AvantGo',
			'BlackBerry',
			'Blazer',
			'Cellphone',
			'Danger',
			'DoCoMo',
			'Elaine/3.0',
			'EudoraWeb',
			'hiptop',
			'iPhone',
			'iPod',
			'MMEF20',
			'MOT-V',
			'NetFront',
			'Newt',
			'Nokia',
			'Opera Mini',
			'Palm',
			'portalmmm',
			'Proxinet',
			'ProxiNet',
			'SHARP-TQ-GX10',
			'Small',
			'SonyEricsson',
			'Symbian OS',
			'SymbianOS',
			'TS21i-10',
			'UP.Browser',
			'UP.Link',
			'Windows CE',
			'WinWAP',
		) );
		natcasesort( $this->browsers );
	}


	// Figure out if the user's browser is a mobile one or not and set a var with our result
	function LookForMobileBrowser() {
		$this->iscompat = ( preg_match( $this->compatregex, $_SERVER['HTTP_USER_AGENT'] ) ) ? TRUE : FALSE;

		// If the user has manually selected the mobile interface, assume they're on a mobile device
		if ( 1 == $_COOKIE[WPHONE_USEUI_COOKIE] ) {
			$this->ismobile = TRUE;
		} else {
			// Check to see if the user's browser is on the browser whitelist
			foreach ( (array) $this->whitelist as $browser ) {
				if ( strstr( $_SERVER['HTTP_USER_AGENT'], $browser ) ) {
					$this->ismobile = FALSE;
					return;
				}
			}

			// Check for known mobile browsers
			foreach ( (array) $this->browsers as $browser ) {
				if ( strstr( $_SERVER['HTTP_USER_AGENT'], $browser ) ) {
					$this->ismobile = TRUE;
					return;
				}
			}

			// Just to be really sure
			$this->ismobile = FALSE;
		}
	}


	// If we're going to display the mobile interface, start capturing the output
	function MaybeStartOutputBuffering() {
		if ( !$this->ismobile ) return;

		$this->obstartstatus = ob_start();
	}


	// If ob_start() failed, display an error message
	function MaybeDisplayBufferError() {
		if ( !$this->ismobile || FALSE != $this->obstartstatus ) return;

		echo '<div id="wphone-error" class="error"><p>' . __('WPhone was unable to display the mobile interface as <code>ob_start()</code> failed to initialize.', 'wphone') . "</p></div>\n\n";
	}


	// We need to modify some redirects to better display data
	function MaybeChangeRedirect( $location ) {
		if ( !$this->ismobile || FALSE == $this->obstartstatus ) return $location;

		global $user_ID;

		// After deleting a comment
		if ( 'comment.php' == $this->current_basename && 'deletecomment' == $_GET['action'] && get_option('siteurl') . '/wp-admin/edit-comments.php' == $location ) {
			$location = 'edit-comments.php?type=approved&parent=edit-comments';
		}

		// After managing a post and it's not published
		elseif ( 'post.php' == $this->current_basename ) {
			$post_ID = (int) $_POST['post_ID'];
			$post = get_post($post_ID);

			// If a draft, redirect to your or others' drafts
			if ( 'post' == $post->post_type && 'draft' == $post->post_status ) {
				$location = ( $user_ID == $post->post_author ) ? 'edit.php?post_status=draft&author=' . $user_ID : 'edit.php?post_status=draft&author=-' . $user_ID;
			}

			// if it's pending, redirect to the pending list
			elseif ( 'post' == $post->post_type && 'pending' == $post->post_status ) {
				$location = 'edit.php?post_status=pending';
			}
		}

		return $location;
	}


	// If the user has a cookie saying they want to use the mobile interface, then take it over
	// An is_admin() check has already been done inside the initalization function
	function MaybeOverride() {
		if ( !$this->ismobile || FALSE == $this->obstartstatus ) return;

		auth_redirect(); // Make sure the user is logged in with credentials

		ob_clean(); // Dump any HTML that's already been created

		$current_basename = $this->current_basename; // Incase the old var is referenced anywhere

		// Figure out which one of files contains the stuff we need as a base
		switch ( $this->current_basename ) {
			// Write or edit posts and pages
			case 'post.php' :
			case 'page.php' :
			case 'post-new.php' :
			case 'page-new.php' :
				$this->context =  ( strstr($this->current_basename, 'page') ) ? 'page' : 'post';
				$include_file  = 'write.php';
				break;

			// List posts or pages
			case 'edit.php' :
			case 'edit-pages.php' :
				$this->context =  ( strstr($this->current_basename, 'page') ) ? 'page' : 'post';
				$include_file  = 'list.php';
				break;

			// List or edit comments
			case 'edit-comments.php' :
			case 'comment.php' :
				$this->context =  ( strstr($this->current_basename, 's.php') ) ? 'list' : 'edit';
				$include_file  = 'comment.php';
				break;

			// List users or edit user (including self)
			case 'profile.php' :
			case 'users.php' :
			case 'user-edit.php' :
				$this->context = ( strstr($this->current_basename, 'users') ) ? 'list' : 'edit';
				$include_file = 'user.php';
				break;

			// List, add or edit categories
			case 'categories.php' :
				$this->context = ( $_GET['action'] ) ? 'edit' : 'list';
				$include_file = 'category.php';
				break;
				
			// Index or unknown page
			default :
				$this->context = 'dashboard';
				$include_file = 'dashboard.php';
		}

		// Special case for successful submit of the "Your Profile" form
		if ( 'profile.php' == $this->current_basename && !empty($_GET['updated']) ) {
			$this->context = 'dashboard';
			$include_file = 'dashboard.php';
			$profileupdated = TRUE;
		}


		global $wpdb, $userdata;

		if ( empty($userdata->ID) ) get_currentuserinfo(); // Just incase

		// Attempt to gzip page to make faster load times
		ob_start( 'ob_gzhandler' );

		// Load up the admin functions and such
		require_once( ABSPATH . 'wp-admin/admin.php');

		// Filters!
		$this->context = apply_filters( 'wphone_context', $this->context );
		$include_file = apply_filters( 'wphone_includefile', ABSPATH . $this->folder . '/includes/' . $include_file );

		// Load the file to display
		if ( !file_exists( $include_file ) ) {
			wp_die( sprintf( __('The file WPhone needed to display this page could not be found. <a href="%s">Click here to disable the mobile interface</a>.', 'wphone'), './?wphone=off' ) );
		}

		require_once( $include_file );

		// Stop from displaying the rest of the stuff below the "admin_footer" hook
		exit();
	}


	// Used to display access denied messages in the interface
	function check_user_permissions() {
		$args = func_get_args();

		if ( ! current_user_can( $args[0], $args[1], $args[2] ) ) {
			die ( 
				'<script type="text/javascript">alert("' . __('Access Denied', 'wphone') . '");</script>'
				. '<div id="errormenu" title="' . __('Error', 'wphone') . '" class="panel">'
				. sprintf( __('You are not allowed to %s.', 'wphone'), $args[0] )
				. '</div>'
			);
		}
	}


	// Returns TRUE if the user is using a mobile browser, otherwise returns FALSE
	// Also accepts a browser identifier to test specifically for
	function is_mobile( $browser = FALSE ) {
		if ( FALSE === $this->ismobile ) return FALSE;

		// Default call to the function (we've already checked to see if we're using a mobile browser)
		if ( FALSE === $browser ) return TRUE;

		// Check for a specific, user-specified browser
		if ( TRUE === $this->ismobile && strstr( $_SERVER['HTTP_USER_AGENT'], $browser ) ) return TRUE;

		// User-specified browser not found
		return FALSE;
	}


	// Returns TRUE if the user is using an iPhone or other high support device, otherwise returns FALSE
	// Also accepts a regex string to test specifically for (use the "wphone_compatregex" filter to filter the default regex)
	function is_mobile_highcompat( $regex = FALSE ) {
		if ( FALSE === $regex ) return $this->iscompat;

		return ( preg_match( $regex, $_SERVER['HTTP_USER_AGENT'] ) ) ? TRUE : FALSE;
	}


	// Loads the requested interface header/footer, if required.
	function load_interface( $part = 'header' ) {
		if ( $part != 'header' ) $part = 'footer'; 
		require_once( apply_filters( 'wphone_loadinterface', ABSPATH . $this->folder . '/includes/' . $part . '.php')  );
	}


	// Loads the requested interface header/footer, if required.
	function test_interface($part = 'header') {
		if ( $part != 'header' ) $part = 'footer'; 
		if ( !$this->iscompat || $_REQUEST['wphone'] != 'ajax' ) {
			$this->load_interface( $part );
		}
	}


	// Output the right kind of button based on browser type
	function panel_button( $type, $name, $value, $onclick = '', $class_change = FALSE )
	{
		if ( !$this->iscompat) {
			echo '<input class="widerfield" type="' . $type . '" name="' . $name . '" id="' . $name . '" value="' . $value . '" /><br />' . "\n";
		} else {
			if ( $onclick == '' && $type == 'submit' ) $onclick = "WPhone.submitForm(this)";

			$pattern = '<button class="whiteButton" type="' . $type . '" name="' . $name . '" id="' . $name . '" onclick="' . $onclick . '" >' . $value . '</button>' . "\n";

			if ( $class_change ) $pattern = str_replace( 'whitebutton', $class_change, $pattern );

			echo $pattern;
		}
	}
	
	
	function panel_delete_button( $value, $question, $url )
	{
		if ( !$this->iscompat) {
			echo '<a href="' .$url. '">' .$value. "</a><br />\n";
		} else {
			$question = js_escape($question);
			$url = js_escape($url);
			$onclick = "if ( confirm('$question') ) { top.location.href = '$url';}";
			echo '<button class="whiteButton" type="button" name="deletebutton" onclick="' . $onclick . '" >' . $value . '</button>' . "\n";
		}
	}


	// WP draft nags, reusable and with customized HTML.
	// Adapted from code in /wp-admin/post-new.php.
	function draft_nags() {
		global $user_ID;

		if ( file_exists( ABSPATH . 'wp-admin/includes/admin.php' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/admin.php' );
		} else {
			require_once( ABSPATH . 'wp-admin/admin-db.php' );
		}

		$my_drafts     = get_users_drafts($user_ID);
		$pending       = ( function_exists('get_others_pending') ) ? get_others_pending($user_ID) : array();
		$others_drafts = ( function_exists('get_others_drafts') ) ? get_others_drafts($user_ID) : array();
		
		$nag_posts = array(
			array(
				'my_drafts',
				__('Your Drafts:'),
				'edit.php?post_status=draft&amp;author=' . $user_ID,
				count($my_drafts)),
			array(
				'pending',
				__('Pending Review:'),
				'edit.php?post_status=pending',
				count($pending)),
			array(
				'others_drafts',
				__('Others&#8217; Drafts:'),
				'edit.php?post_status=draft&author=-' . $user_ID,
				count($others_drafts))
		);

		$html = '';

		if ( !empty($my_drafts) || !empty($pending) || !empty($others_drafts) ) {
			foreach ( $nag_posts as $nag ) {
				if ( ${$nag[0]} ) {
					$html .= sprintf('<li><a href="%s&amp;wphone=ajax">%s %d</a></li>' . "\n", $nag[2], $nag[1], $nag[3]);
				}
			}
		}

		return $html;
	}


	// If referer is not admin area, return it, else return given URL
	function referer( $url = FALSE ) {
		if ( $referer = wp_get_referer() ) {
			if ( !strstr( $referer, 'wp-admin' ) ) return $referer;
		}

		return $url;
	}


	// Outputs a target="blah" tag for the high compat version
	function htmltarget( $target ) {
		if ( !$this->iscompat ) return;
		echo ' target="' . $target . '"';
	}


	// Adds a new checkbox to the login form
	function LoginFormCheckbox() { ?>
	<p><label><input type="checkbox" name="usemobile" id="usemobile" value="1" tabindex="80"<?php if ( 1 == $_COOKIE[WPHONE_CHKBOX_COOKIE] || FALSE !== is_mobile() ) echo ' checked="checked"'; ?> /> <?php _e('Use mobile admin interface', 'wphone'); ?></label></p>
<?php
	}

	
	// When a user logs in, this function checks to see if they wanted to use the mobile interface
	// If they did, then it stores their preferences in cookies
	function LoginFormPOST() {
		// The "wp_login" hook can be triggered when a logged in user visits wp-login.php apparently, so check for $_POST
		if ( 0 < count($_POST) ) {
			// If the user didn't want to use the mobile interface, make sure they don't have our checkbox cookie
			if ( 1 != $_POST['usemobile'] ) {
				setcookie( WPHONE_USEUI_COOKIE, 1, time() - 31536000, COOKIEPATH, COOKIE_DOMAIN );
				setcookie( WPHONE_CHKBOX_COOKIE, 1, time() - 31536000, COOKIEPATH, COOKIE_DOMAIN );
			}

			// The user wants to use the mobile interface. Set a cookie for that and that they checked the checkbox.
			else {
				setcookie( WPHONE_USEUI_COOKIE, 1, time() + 31536000, COOKIEPATH, COOKIE_DOMAIN );
				setcookie( WPHONE_CHKBOX_COOKIE, 1, time() + 31536000, COOKIEPATH, COOKIE_DOMAIN );
			}
		}
	}


	// Deletes this plugin's "use mobile interface" cookie (but not the checkbox preference cookie)
	function ClearCookie() {
		setcookie( WPHONE_USEUI_COOKIE, 0, time() - 31536000, COOKIEPATH, COOKIE_DOMAIN );
	}
}

$WPhone = new WPhone();


// External wrappers
function is_mobile( $browser = FALSE ) {
	global $WPhone;
	return $WPhone->is_mobile( $browser );
}
function is_mobile_highcompat( $regex = FALSE ) {
	global $WPhone;
	return $WPhone->is_mobile_highcompat( $regex );
}

?>